package it.unibs.fp.patenteelibbrettopregoh;

import java.time.LocalDate;
import java.util.ArrayList;

public class GameData {
    private final Formato formato;
    private final String nomePersonaggio;
    private final ArrayList<PaginaDiario> diario;
    private final Comuni comuni;
    private final ArrayList<Persona> persone;
    private double saldo;
    private int numeroMazzetteAccettatePolizia;
    private int numeroRivoluzionariAccettati;
    private StatoGioco statoGioco;
    private int giorno;
    private int indiceUltimaPersonaPassata;
    private LocalDate data;

    public GameData(Formato formato) {
        this.formato = formato;
        this.diario = new ArrayList<>();
        this.comuni = Parser.parseComuni(formato, "FileEsercizio/comuni");
        this.persone = Parser.parsePersone(formato, "FileEsercizio/IDpersone");
        this.saldo = 0;
        this.numeroMazzetteAccettatePolizia = 0;
        this.numeroRivoluzionariAccettati = 0;
        this.statoGioco = StatoGioco.IN_CORSO;
        this.nomePersonaggio = "Ajeje Brazorf";
        this.giorno = 1;
        this.indiceUltimaPersonaPassata = -1;
        this.data = LocalDate.of(2024, 5, giorno);
    }

    public ArrayList<PaginaDiario> getDiario() {
        return diario;
    }

    public Comuni getComuni() {
        return comuni;
    }

    public ArrayList<Persona> getPersone() {
        return persone;
    }

    public double getSaldo() {
        return saldo;
    }

    public Persona getPersonaDaControlare() {
        indiceUltimaPersonaPassata++;
        return persone.get(indiceUltimaPersonaPassata);

    }

    public int getNumeroMazzetteAccettatePolizia() {
        return numeroMazzetteAccettatePolizia;
    }

    public int getNumeroRivoluzionariAccettati() {
        return numeroRivoluzionariAccettati;
    }

    public void addPaginaDiario(PaginaDiario pagina) {
        diario.add(pagina);
    }

    public void addSaldo(double valore) {
        saldo += valore;
    }

    public void pagaMulta(double valore) {
        paga(valore);
    }

    public void incrementaMazzetteAccettatePolizia() {
        numeroMazzetteAccettatePolizia++;
        if (numeroMazzetteAccettatePolizia >= 2) {
            statoGioco = StatoGioco.GIUSTIZIATO;
        }
    }

    public void incrementaRivoluzionariAccettati() {
        numeroRivoluzionariAccettati++;
        if (numeroRivoluzionariAccettati >= 5) {
            statoGioco = StatoGioco.RIVOLUZIONE;
        }
    }

    public String getNomePersonaggio() {
        return nomePersonaggio;
    }

    public StatoGioco getStatoGioco() {
        return statoGioco;
    }

    public int getGiorno() {
        return giorno;
    }

    public void incrementaGiorno(ArrayList<PersonaControllata> personeControllate) {
        if (!personeControllate.isEmpty()) {
            addSaldo(50 * personeControllate.size());
        }
        paga(200);

        PaginaDiario pagina = new PaginaDiario(giorno + "/" + data.getMonthValue() + "/" + data.getYear(), saldo);
        for (PersonaControllata persona : personeControllate) {
            pagina.addPersonaControllata(persona);
        }
        diario.add(pagina);

        giorno++;

        data = LocalDate.of(2024, 5, giorno);

        if (giorno > 12 && statoGioco == StatoGioco.IN_CORSO && saldo > 2300) {
            statoGioco = StatoGioco.DODICESIMO_GIORNO;
        } else if (giorno > 12 && statoGioco == StatoGioco.IN_CORSO) {
            statoGioco = StatoGioco.NON_ABBASTANZA_SOLDI;
        }
    }

    private void paga(double valore) {
        saldo -= valore;
        if (saldo < 0) {
            statoGioco = StatoGioco.ARRESTATO;
        }
    }

    public LocalDate getData() {
        return data;
    }

    public void saveDiario() {
        Parser.saveDiario("FileEsercizio/diario", diario, formato);
    }
}
